# Copyright (c) 2017 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import utils
from kmip.core.messages import payloads


class TestCheckRequestPayload(testtools.TestCase):
    """
    Test suite for the Check request payload.
    """

    def setUp(self):
        super(TestCheckRequestPayload, self).setUp()

        # Encoding obtained in part from the KMIP 1.1 testing document,
        # Section 5.1. The rest of the encoding was built by hand.
        #
        # This encoding matches the following set of values:
        # Request Payload
        #     Unique Identifier - 2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6
        #     Usage Limits Count - 500
        #     Cryptographic Usage Mask - Encrypt | Decrypt (4 | 8 -> 12 or C)
        #     Lease Time - 0

        self.full_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x00\x60'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x32\x63\x32\x33\x32\x31\x37\x65\x2D\x66\x35\x33\x63\x2D\x34\x62'
            b'\x64\x66\x2D\x61\x64\x30\x61\x2D\x35\x38\x61\x33\x31\x66\x64\x33'
            b'\x64\x34\x62\x36\x00\x00\x00\x00'
            b'\x42\x00\x96\x03\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x01\xF4'
            b'\x42\x00\x2C\x02\x00\x00\x00\x04\x00\x00\x00\x0C\x00\x00\x00\x00'
            b'\x42\x00\x49\x0A\x00\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00'
        )

        # This encoding matches the following set of values:
        # Request Payload
        #     Unique Identifier - 2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6
        #     Usage Limits Count - 500
        self.partial_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x00\x40'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x32\x63\x32\x33\x32\x31\x37\x65\x2D\x66\x35\x33\x63\x2D\x34\x62'
            b'\x64\x66\x2D\x61\x64\x30\x61\x2D\x35\x38\x61\x33\x31\x66\x64\x33'
            b'\x64\x34\x62\x36\x00\x00\x00\x00'
            b'\x42\x00\x96\x03\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x01\xF4'
        )

        self.empty_encoding = utils.BytearrayStream(
            b'\x42\x00\x79\x01\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestCheckRequestPayload, self).tearDown()

    def test_init(self):
        """
        Test that a Check request payload can be constructed with no arguments.
        """
        payload = payloads.CheckRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)
        self.assertEqual(None, payload.cryptographic_usage_mask)
        self.assertEqual(None, payload.lease_time)

    def test_init_with_args(self):
        """
        Test that a Check request payload can be constructed with valid values.
        """
        payload = payloads.CheckRequestPayload(
            unique_identifier='00000000-1111-2222-3333-444444444444',
            usage_limits_count=10,
            cryptographic_usage_mask=12,
            lease_time=1000000000
        )

        self.assertEqual(
            '00000000-1111-2222-3333-444444444444',
            payload.unique_identifier
        )
        self.assertEqual(10, payload.usage_limits_count)
        self.assertEqual(12, payload.cryptographic_usage_mask)
        self.assertEqual(1000000000, payload.lease_time)

    def test_invalid_unique_identifier(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the unique identifier of a Check request payload.
        """
        kwargs = {'unique_identifier': 0}
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            payloads.CheckRequestPayload,
            **kwargs
        )

        payload = payloads.CheckRequestPayload()
        args = (payload, 'unique_identifier', 0)
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            setattr,
            *args
        )

    def test_invalid_usage_limits_count(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the usage limits count of a Check request payload.
        """
        kwargs = {'usage_limits_count': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Usage limits count must be an integer.",
            payloads.CheckRequestPayload,
            **kwargs
        )

        payload = payloads.CheckRequestPayload()
        args = (payload, 'usage_limits_count', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "Usage limits count must be an integer.",
            setattr,
            *args
        )

    def test_invalid_cryptographic_usage_mask(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the cryptographic usage mask of a Check request payload.
        """
        kwargs = {'cryptographic_usage_mask': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Cryptographic usage mask must be an integer.",
            payloads.CheckRequestPayload,
            **kwargs
        )

        payload = payloads.CheckRequestPayload()
        args = (payload, 'cryptographic_usage_mask', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "Cryptographic usage mask must be an integer.",
            setattr,
            *args
        )

    def test_invalid_lease_time(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the lease time of a Check request payload.
        """
        kwargs = {'lease_time': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Lease time must be an integer.",
            payloads.CheckRequestPayload,
            **kwargs
        )

        payload = payloads.CheckRequestPayload()
        args = (payload, 'lease_time', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "Lease time must be an integer.",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that a Check request payload can be read from a data stream.
        """
        payload = payloads.CheckRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)
        self.assertEqual(None, payload.cryptographic_usage_mask)
        self.assertEqual(None, payload.lease_time)

        payload.read(self.full_encoding)

        self.assertEqual(
            '2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6',
            payload.unique_identifier
        )
        self.assertEqual(500, payload.usage_limits_count)
        self.assertEqual(12, payload.cryptographic_usage_mask)
        self.assertEqual(0, payload.lease_time)

    def test_read_partial(self):
        """
        Test that a Check request payload can be read from a partial data
        stream.
        """
        payload = payloads.CheckRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)
        self.assertEqual(None, payload.cryptographic_usage_mask)
        self.assertEqual(None, payload.lease_time)

        payload.read(self.partial_encoding)

        self.assertEqual(
            '2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6',
            payload.unique_identifier
        )
        self.assertEqual(500, payload.usage_limits_count)
        self.assertEqual(None, payload.cryptographic_usage_mask)
        self.assertEqual(None, payload.lease_time)

    def test_read_empty(self):
        """
        Test that a Check request payload can be read from an empty data
        stream.
        """
        payload = payloads.CheckRequestPayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)
        self.assertEqual(None, payload.cryptographic_usage_mask)
        self.assertEqual(None, payload.lease_time)

        payload.read(self.empty_encoding)

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)
        self.assertEqual(None, payload.cryptographic_usage_mask)
        self.assertEqual(None, payload.lease_time)

    def test_write(self):
        """
        Test that a Check request payload can be written to a data stream.
        """
        payload = payloads.CheckRequestPayload(
            unique_identifier='2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6',
            usage_limits_count=500,
            cryptographic_usage_mask=12,
            lease_time=0
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_partial(self):
        """
        Test that a partial Check request payload can be written to a data
        stream.
        """
        payload = payloads.CheckRequestPayload(
            unique_identifier='2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6',
            usage_limits_count=500
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.partial_encoding), len(stream))
        self.assertEqual(str(self.partial_encoding), str(stream))

    def test_write_empty(self):
        """
        Test that an empty Check request payload can be written to a data
        stream.
        """
        payload = payloads.CheckRequestPayload()
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.empty_encoding), len(stream))
        self.assertEqual(str(self.empty_encoding), str(stream))

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        Check request payloads with the same data.
        """
        a = payloads.CheckRequestPayload()
        b = payloads.CheckRequestPayload()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a = payloads.CheckRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=200,
            cryptographic_usage_mask=4,
            lease_time=1511882848
        )
        b = payloads.CheckRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=200,
            cryptographic_usage_mask=4,
            lease_time=1511882848
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_unique_identifier(self):
        """
        Test that the equality operator returns False when comparing two
        Check request payloads with different unique identifiers.
        """
        a = payloads.CheckRequestPayload(
            unique_identifier='a'
        )
        b = payloads.CheckRequestPayload(
            unique_identifier='b'
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_usage_limits_count(self):
        """
        Test that the equality operator returns False when comparing two
        Check request payloads with different usage limits counts.
        """
        a = payloads.CheckRequestPayload(
            usage_limits_count=0
        )
        b = payloads.CheckRequestPayload(
            usage_limits_count=1
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_cryptographic_usage_mask(self):
        """
        Test that the equality operator returns False when comparing two
        Check request payloads with different cryptographic usage masks.
        """
        a = payloads.CheckRequestPayload(
            cryptographic_usage_mask=4
        )
        b = payloads.CheckRequestPayload(
            cryptographic_usage_mask=12
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_lease_time(self):
        """
        Test that the equality operator returns False when comparing two
        Check request payloads with different lease times.
        """
        a = payloads.CheckRequestPayload(
            lease_time=0
        )
        b = payloads.CheckRequestPayload(
            lease_time=1511882848
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing two
        Check request payloads with different types.
        """
        a = payloads.CheckRequestPayload()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        Check request payloads with the same data.
        """
        a = payloads.CheckRequestPayload()
        b = payloads.CheckRequestPayload()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = payloads.CheckRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=200,
            cryptographic_usage_mask=4,
            lease_time=1511882848
        )
        b = payloads.CheckRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=200,
            cryptographic_usage_mask=4,
            lease_time=1511882848
        )

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_unique_identifier(self):
        """
        Test that the inequality operator returns True when comparing two
        Check request payloads with different unique identifiers.
        """
        a = payloads.CheckRequestPayload(
            unique_identifier='a'
        )
        b = payloads.CheckRequestPayload(
            unique_identifier='b'
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_usage_limits_count(self):
        """
        Test that the inequality operator returns True when comparing two
        Check request payloads with different usage limits counts.
        """
        a = payloads.CheckRequestPayload(
            usage_limits_count=0
        )
        b = payloads.CheckRequestPayload(
            usage_limits_count=1
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_cryptographic_usage_mask(self):
        """
        Test that the inequality operator returns True when comparing two
        Check request payloads with different cryptographic usage masks.
        """
        a = payloads.CheckRequestPayload(
            cryptographic_usage_mask=4
        )
        b = payloads.CheckRequestPayload(
            cryptographic_usage_mask=12
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_lease_time(self):
        """
        Test that the inequality operator returns True when comparing two
        Check request payloads with different lease times.
        """
        a = payloads.CheckRequestPayload(
            lease_time=0
        )
        b = payloads.CheckRequestPayload(
            lease_time=1511882848
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing two
        Check request payloads with different types.
        """
        a = payloads.CheckRequestPayload()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_repr(self):
        """
        Test that repr can be applied to a Check request payload.
        """
        payload = payloads.CheckRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=1000,
            cryptographic_usage_mask=8,
            lease_time=1511882898
        )
        expected = (
            "CheckRequestPayload("
            "unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038', "
            "usage_limits_count=1000, "
            "cryptographic_usage_mask=8, "
            "lease_time=1511882898)"
        )
        observed = repr(payload)

        self.assertEqual(expected, observed)

    def test_str(self):
        """
        Test that str can be applied to a Check request payload
        """
        payload = payloads.CheckRequestPayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=1000,
            cryptographic_usage_mask=8,
            lease_time=1511882898
        )

        expected = str({
            'unique_identifier': '49a1ca88-6bea-4fb2-b450-7e58802c3038',
            'usage_limits_count': 1000,
            'cryptographic_usage_mask': 8,
            'lease_time': 1511882898
        })
        observed = str(payload)

        self.assertEqual(expected, observed)


class TestCheckResponsePayload(testtools.TestCase):
    """
    Test suite for the Check response payload.
    """

    def setUp(self):
        super(TestCheckResponsePayload, self).setUp()

        # Encoding obtained in part from the KMIP 1.1 testing document,
        # Section 5.1. The rest of the encoding was built by hand.
        #
        # This encoding matches the following set of values:
        # Response Payload
        #     Unique Identifier - 2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6
        #     Usage Limits Count - 500
        #     Cryptographic Usage Mask - Encrypt | Decrypt (4 | 8 -> 12 or C)
        #     Lease Time - 0

        self.full_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x60'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x32\x63\x32\x33\x32\x31\x37\x65\x2D\x66\x35\x33\x63\x2D\x34\x62'
            b'\x64\x66\x2D\x61\x64\x30\x61\x2D\x35\x38\x61\x33\x31\x66\x64\x33'
            b'\x64\x34\x62\x36\x00\x00\x00\x00'
            b'\x42\x00\x96\x03\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x01\xF4'
            b'\x42\x00\x2C\x02\x00\x00\x00\x04\x00\x00\x00\x0C\x00\x00\x00\x00'
            b'\x42\x00\x49\x0A\x00\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00'
        )

        # This encoding matches the following set of values:
        # Response Payload
        #     Unique Identifier - 2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6
        #     Usage Limits Count - 500
        self.partial_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x40'
            b'\x42\x00\x94\x07\x00\x00\x00\x24'
            b'\x32\x63\x32\x33\x32\x31\x37\x65\x2D\x66\x35\x33\x63\x2D\x34\x62'
            b'\x64\x66\x2D\x61\x64\x30\x61\x2D\x35\x38\x61\x33\x31\x66\x64\x33'
            b'\x64\x34\x62\x36\x00\x00\x00\x00'
            b'\x42\x00\x96\x03\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x01\xF4'
        )

        self.empty_encoding = utils.BytearrayStream(
            b'\x42\x00\x7C\x01\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestCheckResponsePayload, self).tearDown()

    def test_init(self):
        """
        Test that a Check response payload can be constructed with no
        arguments.
        """
        payload = payloads.CheckResponsePayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)
        self.assertEqual(None, payload.cryptographic_usage_mask)
        self.assertEqual(None, payload.lease_time)

    def test_init_with_args(self):
        """
        Test that a Check response payload can be constructed with valid
        values.
        """
        payload = payloads.CheckResponsePayload(
            unique_identifier='00000000-1111-2222-3333-444444444444',
            usage_limits_count=10,
            cryptographic_usage_mask=12,
            lease_time=1000000000
        )

        self.assertEqual(
            '00000000-1111-2222-3333-444444444444',
            payload.unique_identifier
        )
        self.assertEqual(10, payload.usage_limits_count)
        self.assertEqual(12, payload.cryptographic_usage_mask)
        self.assertEqual(1000000000, payload.lease_time)

    def test_invalid_unique_identifier(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the unique identifier of a Check response payload.
        """
        kwargs = {'unique_identifier': 0}
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            payloads.CheckResponsePayload,
            **kwargs
        )

        payload = payloads.CheckResponsePayload()
        args = (payload, 'unique_identifier', 0)
        self.assertRaisesRegex(
            TypeError,
            "Unique identifier must be a string.",
            setattr,
            *args
        )

    def test_invalid_usage_limits_count(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the usage limits count of a Check response payload.
        """
        kwargs = {'usage_limits_count': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Usage limits count must be an integer.",
            payloads.CheckResponsePayload,
            **kwargs
        )

        payload = payloads.CheckResponsePayload()
        args = (payload, 'usage_limits_count', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "Usage limits count must be an integer.",
            setattr,
            *args
        )

    def test_invalid_cryptographic_usage_mask(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the cryptographic usage mask of a Check response payload.
        """
        kwargs = {'cryptographic_usage_mask': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Cryptographic usage mask must be an integer.",
            payloads.CheckResponsePayload,
            **kwargs
        )

        payload = payloads.CheckResponsePayload()
        args = (payload, 'cryptographic_usage_mask', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "Cryptographic usage mask must be an integer.",
            setattr,
            *args
        )

    def test_invalid_lease_time(self):
        """
        Test that a TypeError is raised when an invalid value is used to set
        the lease time of a Check response payload.
        """
        kwargs = {'lease_time': 'invalid'}
        self.assertRaisesRegex(
            TypeError,
            "Lease time must be an integer.",
            payloads.CheckResponsePayload,
            **kwargs
        )

        payload = payloads.CheckResponsePayload()
        args = (payload, 'lease_time', 'invalid')
        self.assertRaisesRegex(
            TypeError,
            "Lease time must be an integer.",
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that a Check response payload can be read from a data stream.
        """
        payload = payloads.CheckResponsePayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)
        self.assertEqual(None, payload.cryptographic_usage_mask)
        self.assertEqual(None, payload.lease_time)

        payload.read(self.full_encoding)

        self.assertEqual(
            '2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6',
            payload.unique_identifier
        )
        self.assertEqual(500, payload.usage_limits_count)
        self.assertEqual(12, payload.cryptographic_usage_mask)
        self.assertEqual(0, payload.lease_time)

    def test_read_partial(self):
        """
        Test that a Check response payload can be read from a partial data
        stream.
        """
        payload = payloads.CheckResponsePayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)
        self.assertEqual(None, payload.cryptographic_usage_mask)
        self.assertEqual(None, payload.lease_time)

        payload.read(self.partial_encoding)

        self.assertEqual(
            '2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6',
            payload.unique_identifier
        )
        self.assertEqual(500, payload.usage_limits_count)
        self.assertEqual(None, payload.cryptographic_usage_mask)
        self.assertEqual(None, payload.lease_time)

    def test_read_empty(self):
        """
        Test that a Check response payload can be read from an empty data
        stream.
        """
        payload = payloads.CheckResponsePayload()

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)
        self.assertEqual(None, payload.cryptographic_usage_mask)
        self.assertEqual(None, payload.lease_time)

        payload.read(self.empty_encoding)

        self.assertEqual(None, payload.unique_identifier)
        self.assertEqual(None, payload.usage_limits_count)
        self.assertEqual(None, payload.cryptographic_usage_mask)
        self.assertEqual(None, payload.lease_time)

    def test_write(self):
        """
        Test that a Check response payload can be written to a data stream.
        """
        payload = payloads.CheckResponsePayload(
            unique_identifier='2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6',
            usage_limits_count=500,
            cryptographic_usage_mask=12,
            lease_time=0
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_partial(self):
        """
        Test that a partial Check response payload can be written to a data
        stream.
        """
        payload = payloads.CheckResponsePayload(
            unique_identifier='2c23217e-f53c-4bdf-ad0a-58a31fd3d4b6',
            usage_limits_count=500
        )
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.partial_encoding), len(stream))
        self.assertEqual(str(self.partial_encoding), str(stream))

    def test_write_empty(self):
        """
        Test that an empty Check response payload can be written to a data
        stream.
        """
        payload = payloads.CheckResponsePayload()
        stream = utils.BytearrayStream()
        payload.write(stream)

        self.assertEqual(len(self.empty_encoding), len(stream))
        self.assertEqual(str(self.empty_encoding), str(stream))

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        Check response payloads with the same data.
        """
        a = payloads.CheckResponsePayload()
        b = payloads.CheckResponsePayload()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

        a = payloads.CheckResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=200,
            cryptographic_usage_mask=4,
            lease_time=1511882848
        )
        b = payloads.CheckResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=200,
            cryptographic_usage_mask=4,
            lease_time=1511882848
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal_unique_identifier(self):
        """
        Test that the equality operator returns False when comparing two
        Check response payloads with different unique identifiers.
        """
        a = payloads.CheckResponsePayload(
            unique_identifier='a'
        )
        b = payloads.CheckResponsePayload(
            unique_identifier='b'
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_usage_limits_count(self):
        """
        Test that the equality operator returns False when comparing two
        Check response payloads with different usage limits counts.
        """
        a = payloads.CheckResponsePayload(
            usage_limits_count=0
        )
        b = payloads.CheckResponsePayload(
            usage_limits_count=1
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_cryptographic_usage_mask(self):
        """
        Test that the equality operator returns False when comparing two
        Check response payloads with different cryptographic usage masks.
        """
        a = payloads.CheckResponsePayload(
            cryptographic_usage_mask=4
        )
        b = payloads.CheckResponsePayload(
            cryptographic_usage_mask=12
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_not_equal_lease_time(self):
        """
        Test that the equality operator returns False when comparing two
        Check response payloads with different lease times.
        """
        a = payloads.CheckResponsePayload(
            lease_time=0
        )
        b = payloads.CheckResponsePayload(
            lease_time=1511882848
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing two
        Check response payloads with different types.
        """
        a = payloads.CheckResponsePayload()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing two
        Check response payloads with the same data.
        """
        a = payloads.CheckResponsePayload()
        b = payloads.CheckResponsePayload()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = payloads.CheckResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=200,
            cryptographic_usage_mask=4,
            lease_time=1511882848
        )
        b = payloads.CheckResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=200,
            cryptographic_usage_mask=4,
            lease_time=1511882848
        )

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal_unique_identifier(self):
        """
        Test that the inequality operator returns True when comparing two
        Check response payloads with different unique identifiers.
        """
        a = payloads.CheckResponsePayload(
            unique_identifier='a'
        )
        b = payloads.CheckResponsePayload(
            unique_identifier='b'
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_usage_limits_count(self):
        """
        Test that the inequality operator returns True when comparing two
        Check response payloads with different usage limits counts.
        """
        a = payloads.CheckResponsePayload(
            usage_limits_count=0
        )
        b = payloads.CheckResponsePayload(
            usage_limits_count=1
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_cryptographic_usage_mask(self):
        """
        Test that the inequality operator returns True when comparing two
        Check response payloads with different cryptographic usage masks.
        """
        a = payloads.CheckResponsePayload(
            cryptographic_usage_mask=4
        )
        b = payloads.CheckResponsePayload(
            cryptographic_usage_mask=12
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_not_equal_lease_time(self):
        """
        Test that the inequality operator returns True when comparing two
        Check response payloads with different lease times.
        """
        a = payloads.CheckResponsePayload(
            lease_time=0
        )
        b = payloads.CheckResponsePayload(
            lease_time=1511882848
        )

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing two
        Check response payloads with different types.
        """
        a = payloads.CheckResponsePayload()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_repr(self):
        """
        Test that repr can be applied to a Check response payload.
        """
        payload = payloads.CheckResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=1000,
            cryptographic_usage_mask=8,
            lease_time=1511882898
        )
        expected = (
            "CheckResponsePayload("
            "unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038', "
            "usage_limits_count=1000, "
            "cryptographic_usage_mask=8, "
            "lease_time=1511882898)"
        )
        observed = repr(payload)

        self.assertEqual(expected, observed)

    def test_str(self):
        """
        Test that str can be applied to a Check response payload
        """
        payload = payloads.CheckResponsePayload(
            unique_identifier='49a1ca88-6bea-4fb2-b450-7e58802c3038',
            usage_limits_count=1000,
            cryptographic_usage_mask=8,
            lease_time=1511882898
        )

        expected = str({
            'unique_identifier': '49a1ca88-6bea-4fb2-b450-7e58802c3038',
            'usage_limits_count': 1000,
            'cryptographic_usage_mask': 8,
            'lease_time': 1511882898
        })
        observed = str(payload)

        self.assertEqual(expected, observed)



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from kmip.core import enums
from kmip.core.enums import KeyFormatType as KeyFormatTypeEnum
from kmip.core.enums import Tags
from kmip.core.enums import QueryFunction as QueryFunctionEnum

from kmip.core.primitives import ByteString
from kmip.core.primitives import Enumeration
from kmip.core.primitives import Interval
from kmip.core.primitives import Struct
from kmip.core.primitives import TextString

from kmip.core.utils import BytearrayStream


class CertificateValue(ByteString):
    """
    The bytes of a DER-encoded X.509 public key certificate.

    Used by the Certificate Managed Object to store the bytes of the
    certificate. See Section 2.2.1 of the KMIP 1.1. specification for more
    information.
    """

    def __init__(self, value=b''):
        """
        Construct a CertificateValue byte string.

        Args:
            value (bytes): A byte string (e.g., b'\x00\x01...') containing the
                certificate bytes to store. Optional, defaults to the empty
                byte string.
        """
        super(CertificateValue, self).__init__(value, Tags.CERTIFICATE_VALUE)


class Offset(Interval):
    """
    An integer representing a positive change in time.

    Used by Rekey and Recertify requests to indicate the time difference
    between the InitializationDate and the ActivationDate of the replacement
    item to be created. See Sections 4.4, 4.5, and 4.8 of the KMIP 1.1
    specification for more information.
    """

    def __init__(self, value=None):
        """
        Construct an Offset object.

        Args:
            value (int): An integer representing a positive change in time.
                Optional, defaults to None.
        """
        super(Offset, self).__init__(value, Tags.OFFSET)


class QueryFunction(Enumeration):
    """
    An encodeable wrapper for the QueryFunction enumeration.

    Used by Query requests to specify the information to retrieve from the
    KMIP server. See Sections 4.25 and 9.1.3.2.24 of the KMIP 1.1
    specification for more information.
    """

    def __init__(self, value=None):
        """
        Construct a QueryFunction object.

        Args:
            value (QueryFunction enum): A QueryFunction enumeration value,
                (e.g., QueryFunction.QUERY_OPERATIONS). Optional, default to
                None.
        """
        super(QueryFunction, self).__init__(
            QueryFunctionEnum, value, Tags.QUERY_FUNCTION)


class VendorIdentification(TextString):
    """
    A text string uniquely identifying a KMIP vendor.

    Returned by KMIP servers upon receipt of a Query request for server
    information. See Section 4.25 of the KMIP 1.1. specification for more
    information.
    """

    def __init__(self, value=None):
        """
        Construct a VendorIdentification object.

        Args:
            value (str): A string describing a KMIP vendor. Optional, defaults
                to None.
        """
        super(VendorIdentification, self).__init__(
            value, Tags.VENDOR_IDENTIFICATION)


class ServerInformation(Struct):
    """
    A structure containing vendor-specific fields and/or substructures.

    Returned by KMIP servers upon receipt of a Query request for server
    information. See Section 4.25 of the KMIP 1.1 specification for more
    information.

    Note:
    There are no example structures nor data encodings in the KMIP
    documentation of this object. Therefore this class handles encoding and
    decoding its data in a generic way, using a BytearrayStream for primary
    storage. The intent is for vendor-specific subclasses to decide how to
    decode this data from the stream attribute. Likewise, these subclasses
    must decide how to encode their data into the stream attribute. There
    are no arguments to the constructor and therefore no means by which to
    validate the object's contents.
    """

    def __init__(self):
        """
        Construct a ServerInformation object.
        """
        super(ServerInformation, self).__init__(Tags.SERVER_INFORMATION)

        self.data = BytearrayStream()

        self.validate()

    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Read the data encoding the ServerInformation object and decode it into
        its constituent parts.

        Args:
            istream (Stream): A data stream containing encoded object data,
                supporting a read method; usually a BytearrayStream object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be decoded. Optional,
                defaults to KMIP 1.0.
        """
        super(ServerInformation, self).read(istream, kmip_version=kmip_version)
        tstream = BytearrayStream(istream.read(self.length))

        self.data = BytearrayStream(tstream.read())

        self.is_oversized(tstream)
        self.validate()

    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        """
        Write the data encoding the ServerInformation object to a stream.

        Args:
            ostream (Stream): A data stream in which to encode object data,
                supporting a write method; usually a BytearrayStream object.
            kmip_version (KMIPVersion): An enumeration defining the KMIP
                version with which the object will be encoded. Optional,
                defaults to KMIP 1.0.
        """
        tstream = BytearrayStream()
        tstream.write(self.data.buffer)

        self.length = tstream.length()
        super(ServerInformation, self).write(
            ostream,
            kmip_version=kmip_version
        )
        ostream.write(tstream.buffer)

    def validate(self):
        """
        Error check the types of the different parts of the ServerInformation
        object.
        """
        self.__validate()

    def __validate(self):
        # NOTE (peter-hamilton): Intentional pass, no way to validate data.
        pass

    def __eq__(self, other):
        if isinstance(other, ServerInformation):
            if len(self.data) != len(other.data):
                return False
            elif self.data != other.data:
                return False
            else:
                return True
        else:
            return NotImplemented

    def __ne__(self, other):
        if isinstance(other, ServerInformation):
            return not (self == other)
        else:
            return NotImplemented

    def __repr__(self):
        return "ServerInformation()"

    def __str__(self):
        return str(self.data)


class KeyFormatType(Enumeration):
    """
    An encodeable wrapper for the KeyFormatType enumeration.

    Used to identify the format of different types of keys in KeyBlock and
    Digest objects, it can also be used to specify the format in which a key
    is returned when using the Get operation. See Sections 2.1.3, 2.1.7, 3.17,
    4.11, and 9.1.3.2.3 of the KMIP 1.1 specification for more information.
    """

    def __init__(self, value=KeyFormatTypeEnum.RAW):
        """
        Construct a KeyFormatType object.

        Args:
            value (KeyFormatType): A KeyFormatType enumeration value,
                (e.g., KeyFormatType.PKCS_1). Optional, default to
                KeyFormatType.RAW.
        """
        super(KeyFormatType, self).__init__(
            KeyFormatTypeEnum, value, Tags.KEY_FORMAT_TYPE)



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from kmip.core import attributes
from kmip.core import misc
from kmip.core import objects

from kmip.core import enums
from kmip.core.messages.payloads import base
from kmip.core.messages.payloads.create_key_pair import \
    CreateKeyPairResponsePayload
from kmip.core.utils import BytearrayStream


class RekeyKeyPairRequestPayload(base.RequestPayload):

    def __init__(self,
                 private_key_uuid=None,
                 offset=None,
                 common_template_attribute=None,
                 private_key_template_attribute=None,
                 public_key_template_attribute=None):
        super(RekeyKeyPairRequestPayload, self).__init__()

        self.private_key_uuid = private_key_uuid
        self.offset = offset
        self.common_template_attribute = common_template_attribute
        self.private_key_template_attribute = private_key_template_attribute
        self.public_key_template_attribute = public_key_template_attribute

        self.validate()

    def read(self, istream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        super(RekeyKeyPairRequestPayload, self).read(
            istream,
            kmip_version=kmip_version
        )
        tstream = BytearrayStream(istream.read(self.length))

        if self.is_tag_next(enums.Tags.PRIVATE_KEY_UNIQUE_IDENTIFIER, tstream):
            self.private_key_uuid = attributes.PrivateKeyUniqueIdentifier()
            self.private_key_uuid.read(tstream, kmip_version=kmip_version)

        if self.is_tag_next(enums.Tags.OFFSET, tstream):
            self.offset = misc.Offset()
            self.offset.read(tstream, kmip_version=kmip_version)

        if self.is_tag_next(enums.Tags.COMMON_TEMPLATE_ATTRIBUTE, tstream):
            self.common_template_attribute = objects.CommonTemplateAttribute()
            self.common_template_attribute.read(
                tstream,
                kmip_version=kmip_version
            )

        if self.is_tag_next(enums.Tags.PRIVATE_KEY_TEMPLATE_ATTRIBUTE,
                            tstream):
            self.private_key_template_attribute = \
                objects.PrivateKeyTemplateAttribute()
            self.private_key_template_attribute.read(
                tstream,
                kmip_version=kmip_version
            )

        if self.is_tag_next(enums.Tags.PUBLIC_KEY_TEMPLATE_ATTRIBUTE, tstream):
            self.public_key_template_attribute = \
                objects.PublicKeyTemplateAttribute()
            self.public_key_template_attribute.read(
                tstream,
                kmip_version=kmip_version
            )

        self.is_oversized(tstream)
        self.validate()

    def write(self, ostream, kmip_version=enums.KMIPVersion.KMIP_1_0):
        tstream = BytearrayStream()

        if self.private_key_uuid is not None:
            self.private_key_uuid.write(tstream, kmip_version=kmip_version)

        if self.offset is not None:
            self.offset.write(tstream, kmip_version=kmip_version)

        if self.common_template_attribute is not None:
            self.common_template_attribute.write(
                tstream,
                kmip_version=kmip_version
            )

        if self.private_key_template_attribute is not None:
            self.private_key_template_attribute.write(
                tstream,
                kmip_version=kmip_version
            )

        if self.public_key_template_attribute is not None:
            self.public_key_template_attribute.write(
                tstream,
                kmip_version=kmip_version
            )

        self.length = tstream.length()
        super(RekeyKeyPairRequestPayload, self).write(
            ostream,
            kmip_version=kmip_version
        )
        ostream.write(tstream.buffer)

    def validate(self):
        self.__validate()

    def __validate(self):
        if self.private_key_uuid is not None:
            if not isinstance(self.private_key_uuid,
                              attributes.PrivateKeyUniqueIdentifier):
                msg = "invalid private key unique identifier"
                msg += "; expected {0}, received {1}".format(
                    attributes.PrivateKeyUniqueIdentifier,
                    self.private_key_uuid)
                raise TypeError(msg)

        if self.offset is not None:
            if not isinstance(self.offset, misc.Offset):
                msg = "invalid offset"
                msg += "; expected {0}, received {1}".format(
                    misc.Offset, self.offset)
                raise TypeError(msg)

        if self.common_template_attribute is not None:
            if not isinstance(self.common_template_attribute,
                              objects.CommonTemplateAttribute):
                msg = "invalid common template attribute"
                msg += "; expected {0}, received {1}".format(
                    objects.CommonTemplateAttribute,
                    self.common_template_attribute)
                raise TypeError(msg)

        if self.private_key_template_attribute is not None:
            if not isinstance(self.private_key_template_attribute,
                              objects.PrivateKeyTemplateAttribute):
                msg = "invalid private key template attribute"
                msg += "; expected {0}, received {1}".format(
                    objects.PrivateKeyTemplateAttribute,
                    self.private_key_template_attribute)
                raise TypeError(msg)

        if self.public_key_template_attribute is not None:
            if not isinstance(self.public_key_template_attribute,
                              objects.PublicKeyTemplateAttribute):
                msg = "invalid public key template attribute"
                msg += "; expected {0}, received {1}".format(
                    objects.PublicKeyTemplateAttribute,
                    self.public_key_template_attribute)
                raise TypeError(msg)


# TODO (ph) Remove the dependency on the CreateKeyPairResponsePayload
class RekeyKeyPairResponsePayload(CreateKeyPairResponsePayload):

    def __init__(self,
                 private_key_uuid=None,
                 public_key_uuid=None,
                 private_key_template_attribute=None,
                 public_key_template_attribute=None):
        super(RekeyKeyPairResponsePayload, self).__init__(
            private_key_uuid, public_key_uuid, private_key_template_attribute,
            public_key_template_attribute)



# Copyright (c) 2016 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pytest
import ssl
import testtools

from kmip.services import auth


class TestBasicAuthenticationSuite(testtools.TestCase):
    """
    A test suite for the BasicAuthenticationSuite.
    """

    def setUp(self):
        super(TestBasicAuthenticationSuite, self).setUp()

    def tearDown(self):
        super(TestBasicAuthenticationSuite, self).tearDown()

    def test_init(self):
        auth.BasicAuthenticationSuite()

    def test_protocol(self):
        suite = auth.BasicAuthenticationSuite()
        protocol = suite.protocol

        self.assertIsInstance(protocol, int)
        self.assertEqual(ssl.PROTOCOL_TLSv1, suite.protocol)

    def test_ciphers(self):
        suite = auth.BasicAuthenticationSuite()
        ciphers = suite.ciphers

        self.assertIsInstance(ciphers, str)

        cipher_string = ':'.join((
            'AES128-SHA',
            'DES-CBC3-SHA',
            'AES256-SHA',
            'DHE-DSS-DES-CBC3-SHA',
            'DHE-RSA-DES-CBC3-SHA',
            'DH-DSS-AES128-SHA',
            'DH-RSA-AES128-SHA',
            'DHE-DSS-AES128-SHA',
            'DHE-RSA-AES128-SHA',
            'DH-RSA-AES256-SHA',
            'DHE-DSS-AES256-SHA',
            'DHE-RSA-AES256-SHA',
        ))

        self.assertEqual(cipher_string, ciphers)

    def test_custom_ciphers(self):
        """
        Test that providing a custom list of cipher suites yields the right
        cipher string for the Basic auth suite.
        """
        suite = auth.BasicAuthenticationSuite(
            [
                'TLS_RSA_WITH_AES_128_CBC_SHA',
                'TLS_RSA_WITH_AES_256_CBC_SHA',
                'TLS_DHE_PSK_WITH_AES_128_CBC_SHA',
                'TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA',
                'DHE-DSS-AES256-SHA',
                'DHE-RSA-AES256-SHA'
            ]
        )
        ciphers = suite.ciphers

        self.assertIsInstance(ciphers, str)
        suites = ciphers.split(':')
        self.assertEqual(4, len(suites))
        self.assertIn('AES128-SHA', suites)
        self.assertIn('AES256-SHA', suites)
        self.assertIn('DHE-DSS-AES256-SHA', suites)
        self.assertIn('DHE-RSA-AES256-SHA', suites)

    def test_custom_ciphers_empty(self):
        """
        Test that providing a custom list of cipher suites that ultimately
        yields an empty suite list causes the default cipher suite list to
        be provided instead.
        """
        suite = auth.BasicAuthenticationSuite(
            [
                'TLS_RSA_WITH_AES_256_CBC_SHA256'
            ]
        )
        ciphers = suite.ciphers

        self.assertIsInstance(ciphers, str)
        suites = ciphers.split(':')
        self.assertEqual(12, len(suites))
        self.assertIn('AES128-SHA', suites)
        self.assertIn('DES-CBC3-SHA', suites)
        self.assertIn('AES256-SHA', suites)
        self.assertIn('DHE-DSS-DES-CBC3-SHA', suites)
        self.assertIn('DHE-RSA-DES-CBC3-SHA', suites)
        self.assertIn('DH-DSS-AES128-SHA', suites)
        self.assertIn('DH-RSA-AES128-SHA', suites)
        self.assertIn('DHE-DSS-AES128-SHA', suites)
        self.assertIn('DHE-RSA-AES128-SHA', suites)
        self.assertIn('DH-RSA-AES256-SHA', suites)
        self.assertIn('DHE-DSS-AES256-SHA', suites)
        self.assertIn('DHE-RSA-AES256-SHA', suites)


@pytest.mark.skipif(not hasattr(ssl, 'PROTOCOL_TLSv1_2'),
                    reason="Requires ssl.PROTOCOL_TLSv1_2")
class TestTLS12AuthenticationSuite(testtools.TestCase):
    """
    A test suite for the TLS12AuthenticationSuite.
    """

    def setUp(self):
        super(TestTLS12AuthenticationSuite, self).setUp()

    def tearDown(self):
        super(TestTLS12AuthenticationSuite, self).tearDown()

    def test_init(self):
        auth.TLS12AuthenticationSuite()

    def test_protocol(self):
        suite = auth.TLS12AuthenticationSuite()
        protocol = suite.protocol

        self.assertIsInstance(protocol, int)
        self.assertEqual(ssl.PROTOCOL_TLSv1_2, suite.protocol)

    def test_ciphers(self):
        suite = auth.TLS12AuthenticationSuite()
        ciphers = suite.ciphers

        self.assertIsInstance(ciphers, str)

        cipher_string = ':'.join((
            'AES128-SHA256',
            'AES256-SHA256',
            'DH-DSS-AES256-SHA256',
            'DH-DSS-AES128-SHA256',
            'DH-RSA-AES128-SHA256',
            'DHE-DSS-AES128-SHA256',
            'DHE-RSA-AES128-SHA256',
            'DH-DSS-AES256-SHA256',
            'DH-RSA-AES256-SHA256',
            'DHE-DSS-AES256-SHA256',
            'DHE-RSA-AES256-SHA256',
            'ECDH-ECDSA-AES128-SHA256',
            'ECDH-ECDSA-AES256-SHA256',
            'ECDHE-ECDSA-AES128-SHA256',
            'ECDHE-ECDSA-AES256-SHA384',
            'ECDH-RSA-AES128-SHA256',
            'ECDH-RSA-AES256-SHA384',
            'ECDHE-RSA-AES128-SHA256',
            'ECDHE-RSA-AES256-SHA384',
            'ECDHE-ECDSA-AES128-GCM-SHA256',
            'ECDHE-ECDSA-AES256-GCM-SHA384',
            'ECDHE-ECDSA-AES128-SHA256',
            'ECDHE-ECDSA-AES256-SHA384',
        ))

        self.assertEqual(cipher_string, ciphers)

    def test_custom_ciphers(self):
        """
        Test that providing a custom list of cipher suites yields the right
        cipher string.
        """
        suite = auth.TLS12AuthenticationSuite(
            [
                'TLS_RSA_WITH_AES_256_CBC_SHA256',
                'TLS_RSA_WITH_AES_256_CBC_SHA',
                'TLS_DHE_PSK_WITH_AES_128_CBC_SHA',
                'TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA',
                'DHE-DSS-AES256-SHA',
                'DHE-RSA-AES256-SHA'
            ]
        )
        ciphers = suite.ciphers

        self.assertIsInstance(ciphers, str)
        suites = ciphers.split(':')
        self.assertEqual(1, len(suites))
        self.assertIn('AES256-SHA256', suites)

    def test_custom_ciphers_empty(self):
        """
        Test that providing a custom list of cipher suites that ultimately
        yields an empty suite list causes the default cipher suite list to
        be provided instead.
        """
        suite = auth.TLS12AuthenticationSuite(
            [
                'TLS_RSA_WITH_AES_256_CBC_SHA'
            ]
        )
        ciphers = suite.ciphers

        self.assertIsInstance(ciphers, str)
        suites = ciphers.split(':')
        self.assertEqual(23, len(suites))
        self.assertIn('AES128-SHA256', suites)
        self.assertIn('AES256-SHA256', suites)
        self.assertIn('DH-DSS-AES256-SHA256', suites)
        self.assertIn('DH-DSS-AES128-SHA256', suites)
        self.assertIn('DH-RSA-AES128-SHA256', suites)
        self.assertIn('DHE-DSS-AES128-SHA256', suites)
        self.assertIn('DHE-RSA-AES128-SHA256', suites)
        self.assertIn('DH-DSS-AES256-SHA256', suites)
        self.assertIn('DH-RSA-AES256-SHA256', suites)
        self.assertIn('DHE-DSS-AES256-SHA256', suites)
        self.assertIn('DHE-RSA-AES256-SHA256', suites)
        self.assertIn('ECDH-ECDSA-AES128-SHA256', suites)
        self.assertIn('ECDH-ECDSA-AES256-SHA256', suites)
        self.assertIn('ECDHE-ECDSA-AES128-SHA256', suites)
        self.assertIn('ECDHE-ECDSA-AES256-SHA384', suites)
        self.assertIn('ECDH-RSA-AES128-SHA256', suites)
        self.assertIn('ECDH-RSA-AES256-SHA384', suites)
        self.assertIn('ECDHE-RSA-AES128-SHA256', suites)
        self.assertIn('ECDHE-RSA-AES256-SHA384', suites)
        self.assertIn('ECDHE-ECDSA-AES128-GCM-SHA256', suites)
        self.assertIn('ECDHE-ECDSA-AES256-GCM-SHA384', suites)
        self.assertIn('ECDHE-ECDSA-AES128-SHA256', suites)
        self.assertIn('ECDHE-ECDSA-AES256-SHA384', suites)



# Copyright (c) 2016 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import os
import six

from six.moves import configparser

from kmip.core import exceptions


class KmipServerConfig(object):
    """
    A configuration management tool for the KmipServer.
    """

    def __init__(self):
        """
        Create a KmipServerConfig object.
        """
        self._logger = logging.getLogger('kmip.server.config')

        self.settings = dict()
        self.settings['enable_tls_client_auth'] = True
        self.settings['tls_cipher_suites'] = []
        self.settings['logging_level'] = logging.INFO
        self.settings['auth_plugins'] = []

        self._expected_settings = [
            'hostname',
            'port',
            'certificate_path',
            'key_path',
            'ca_path',
            'auth_suite'
        ]
        self._optional_settings = [
            'policy_path',
            'enable_tls_client_auth',
            'tls_cipher_suites',
            'logging_level',
            'database_path'
        ]

    def set_setting(self, setting, value):
        """
        Set a specific setting value.

        This will overwrite the current setting value for the specified
        setting.

        Args:
            setting (string): The name of the setting to set (e.g.,
                'certificate_path', 'hostname'). Required.
            value (misc): The value of the setting to set. Type varies based
                on setting. Required.
        Raises:
            ConfigurationError: Raised if the setting is not supported or if
                the setting value is invalid.
        """
        if setting not in self._expected_settings + self._optional_settings:
            raise exceptions.ConfigurationError(
                "Setting '{0}' is not supported.".format(setting)
            )

        if setting == 'hostname':
            self._set_hostname(value)
        elif setting == 'port':
            self._set_port(value)
        elif setting == 'certificate_path':
            self._set_certificate_path(value)
        elif setting == 'key_path':
            self._set_key_path(value)
        elif setting == 'ca_path':
            self._set_ca_path(value)
        elif setting == 'auth_suite':
            self._set_auth_suite(value)
        elif setting == 'policy_path':
            self._set_policy_path(value)
        elif setting == 'enable_tls_client_auth':
            self._set_enable_tls_client_auth(value)
        elif setting == 'tls_cipher_suites':
            self._set_tls_cipher_suites(value)
        elif setting == 'logging_level':
            self._set_logging_level(value)
        else:
            self._set_database_path(value)

    def load_settings(self, path):
        """
        Load configuration settings from the file pointed to by path.

        This will overwrite all current setting values.

        Args:
            path (string): The path to the configuration file containing
                the settings to load. Required.
        Raises:
            ConfigurationError: Raised if the path does not point to an
                existing file or if a setting value is invalid.
        """
        if not os.path.exists(path):
            raise exceptions.ConfigurationError(
                "The server configuration file ('{0}') could not be "
                "located.".format(path)
            )

        self._logger.info(
            "Loading server configuration settings from: {0}".format(path)
        )

        parser = configparser.ConfigParser()
        parser.read(path)
        self._parse_settings(parser)
        self.parse_auth_settings(parser)

    def parse_auth_settings(self, parser):
        sections = [x for x in parser.sections() if x.startswith("auth:")]
        configs = [(x, dict(parser.items(x))) for x in sections]
        self.settings['auth_plugins'] = configs

    def _parse_settings(self, parser):
        if not parser.has_section('server'):
            raise exceptions.ConfigurationError(
                "The server configuration file does not have a 'server' "
                "section."
            )

        settings = [x[0] for x in parser.items('server')]
        for s in settings:
            if s not in self._expected_settings + self._optional_settings:
                raise exceptions.ConfigurationError(
                    "Setting '{0}' is not a supported setting. Please "
                    "remove it from the configuration file.".format(s)
                )
        for setting in self._expected_settings:
            if setting not in settings:
                raise exceptions.ConfigurationError(
                    "Setting '{0}' is missing from the configuration "
                    "file.".format(setting)
                )

        if parser.has_option('server', 'hostname'):
            self._set_hostname(parser.get('server', 'hostname'))
        if parser.has_option('server', 'port'):
            self._set_port(parser.getint('server', 'port'))
        if parser.has_option('server', 'certificate_path'):
            self._set_certificate_path(parser.get(
                'server',
                'certificate_path')
            )
        if parser.has_option('server', 'key_path'):
            self._set_key_path(parser.get('server', 'key_path'))
        if parser.has_option('server', 'ca_path'):
            self._set_ca_path(parser.get('server', 'ca_path'))
        if parser.has_option('server', 'auth_suite'):
            self._set_auth_suite(parser.get('server', 'auth_suite'))
        if parser.has_option('server', 'policy_path'):
            self._set_policy_path(parser.get('server', 'policy_path'))
        if parser.has_option('server', 'enable_tls_client_auth'):
            self._set_enable_tls_client_auth(
                parser.getboolean('server', 'enable_tls_client_auth')
            )
        if parser.has_option('server', 'tls_cipher_suites'):
            self._set_tls_cipher_suites(
                parser.get('server', 'tls_cipher_suites')
            )
        if parser.has_option('server', 'logging_level'):
            self._set_logging_level(
                parser.get('server', 'logging_level')
            )
        if parser.has_option('server', 'database_path'):
            self._set_database_path(parser.get('server', 'database_path'))

    def _set_hostname(self, value):
        if isinstance(value, six.string_types):
            self.settings['hostname'] = value
        else:
            raise exceptions.ConfigurationError(
                "The hostname value must be a string."
            )

    def _set_port(self, value):
        if isinstance(value, six.integer_types):
            if 0 < value < 65535:
                self.settings['port'] = value
            else:
                raise exceptions.ConfigurationError(
                    "The port value must be an integer in the range 0 - 65535."
                )
        else:
            raise exceptions.ConfigurationError(
                "The port value must be an integer in the range 0 - 65535."
            )

    def _set_certificate_path(self, value):
        if value is None:
            self.settings['certificate_path'] = None
        elif isinstance(value, six.string_types):
            if os.path.exists(value):
                self.settings['certificate_path'] = value
            else:
                raise exceptions.ConfigurationError(
                    "The certificate path value, if specified, must be a "
                    "valid string path to a certificate file."
                )
        else:
            raise exceptions.ConfigurationError(
                "The certificate path value, if specified, must be a valid "
                "string path to a certificate file."
            )

    def _set_key_path(self, value):
        if value is None:
            self.settings['key_path'] = None
        elif isinstance(value, six.string_types):
            if os.path.exists(value):
                self.settings['key_path'] = value
            else:
                raise exceptions.ConfigurationError(
                    "The key path value, if specified, must be a valid string "
                    "path to a certificate key file."
                )
        else:
            raise exceptions.ConfigurationError(
                "The key path value, if specified, must be a valid string "
                "path to a certificate key file."
            )

    def _set_ca_path(self, value):
        if value is None:
            self.settings['ca_path'] = None
        elif isinstance(value, six.string_types):
            if os.path.exists(value):
                self.settings['ca_path'] = value
            else:
                raise exceptions.ConfigurationError(
                    "The certificate authority (CA) path value, if "
                    "specified, must be a valid string path to a CA "
                    "certificate file."
                )
        else:
            raise exceptions.ConfigurationError(
                "The certificate authority (CA) path value, if specified, "
                "must be a valid string path to a CA certificate file."
            )

    def _set_auth_suite(self, value):
        auth_suites = ['Basic', 'TLS1.2']
        if value not in auth_suites:
            raise exceptions.ConfigurationError(
                "The authentication suite must be one of the "
                "following: Basic, TLS1.2"
            )
        else:
            self.settings['auth_suite'] = value

    def _set_policy_path(self, value):
        if not value:
            self.settings['policy_path'] = None
        elif isinstance(value, six.string_types):
            self.settings['policy_path'] = value
        else:
            raise exceptions.ConfigurationError(
                "The policy path, if specified, must be a valid string path "
                "to a filesystem directory."
            )

    def _set_enable_tls_client_auth(self, value):
        if value is None:
            self.settings['enable_tls_client_auth'] = True
        elif isinstance(value, bool):
            self.settings['enable_tls_client_auth'] = value
        else:
            raise exceptions.ConfigurationError(
                "The flag enabling the TLS certificate client auth flag check "
                "must be a boolean."
            )

    def _set_tls_cipher_suites(self, value):
        if not value:
            self.settings['tls_cipher_suites'] = []
            return
        if isinstance(value, six.string_types):
            value = value.split()

        if isinstance(value, list):
            for entry in value:
                if not isinstance(entry, six.string_types):
                    raise exceptions.ConfigurationError(
                        "The TLS cipher suites must be a set of strings "
                        "representing cipher suite names."
                    )
            self.settings['tls_cipher_suites'] = list(set(value))
        else:
            raise exceptions.ConfigurationError(
                "The TLS cipher suites must be a set of strings representing "
                "cipher suite names."
            )

    def _set_logging_level(self, value):
        if value is None:
            self.settings['logging_level'] = logging.INFO
            return

        logging_levels = {
            "DEBUG":    logging.DEBUG,
            "INFO":     logging.INFO,
            "WARNING":  logging.WARNING,
            "ERROR":    logging.ERROR,
            "CRITICAL": logging.CRITICAL
        }
        if value in logging_levels.values():
            self.settings['logging_level'] = value
        elif isinstance(value, six.string_types):
            level = logging_levels.get(value.upper())
            if level:
                self.settings['logging_level'] = level
            else:
                raise exceptions.ConfigurationError(
                    "The logging level must be a string representing a valid "
                    "logging level."
                )
        else:
            raise exceptions.ConfigurationError(
                "The logging level must be a string representing a valid "
                "logging level."
            )

    def _set_database_path(self, value):
        if not value:
            self.settings['database_path'] = None
        elif isinstance(value, six.string_types):
            self.settings['database_path'] = value
        else:
            raise exceptions.ConfigurationError(
                "The database path, if specified, must be a valid path to a "
                "SQLite database file."
            )



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pytest

from kmip.services import kmip_client
from kmip.pie import client as pclient


def pytest_addoption(parser):
    parser.addoption(
        "--config",
        action="store",
        default="client",
        help="Config file section name for client configuration settings")


@pytest.fixture(scope="class")
def client(request):
    config = request.config.getoption("--config")

    client = kmip_client.KMIPProxy(config=config)
    client.open()

    def finalize():
        client.close()

    request.addfinalizer(finalize)
    request.cls.client = client


@pytest.fixture(scope="class")
def simple(request):
    config = request.config.getoption("--config")

    client = pclient.ProxyKmipClient(config=config)
    client.open()

    def finalize():
        client.close()

    request.addfinalizer(finalize)
    request.cls.client = client



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from testtools import TestCase

from kmip.core import exceptions
from kmip.core import utils


class TestUtils(TestCase):

    def setUp(self):
        super(TestUtils, self).setUp()

    def tearDown(self):
        super(TestUtils, self).tearDown()

    def test_count_bytes(self):
        num = 65535
        bytes_exp = 2
        bytes_obs = utils.count_bytes(num)
        self.assertEqual(bytes_exp, bytes_obs,
                         'Value {0} requires {1} bytes to encode, '
                         'received {2} byte(s)'.format(num, bytes_exp,
                                                       bytes_obs))

    def test_count_bytes_overflow(self):
        num = 65536
        bytes_exp = 3
        bytes_obs = utils.count_bytes(num)
        self.assertEqual(bytes_exp, bytes_obs,
                         'Value {0} requires {1} bytes to encode, '
                         'received {2} bytes'.format(num, bytes_exp,
                                                     bytes_obs))

    def test_count_bytes_zero(self):
        num = 0
        bytes_exp = 1
        bytes_obs = utils.count_bytes(num)
        self.assertEqual(bytes_exp, bytes_obs,
                         'Value {0} requires {1} bytes to encode, '
                         'received {2} byte(s)'.format(num, bytes_exp,
                                                       bytes_obs))


class TestBytearrayStream(TestCase):

    def setUp(self):
        super(TestBytearrayStream, self).setUp()
        self.stream = utils.BytearrayStream()

        self.bad_type = exceptions.ErrorStrings.BAD_EXP_RECV.format(
            'BytearrayStream.{0}',
            'type',
            '{1}',
            '{2}'
        )
        self.bad_len = exceptions.ErrorStrings.BAD_EXP_RECV.format(
            'BytearrayStream.{0}',
            'length',
            '{1}',
            '{2}'
        )
        self.bad_val = exceptions.ErrorStrings.BAD_EXP_RECV.format(
            'BytearrayStream.{0}',
            'value',
            '{1}',
            '{2}'
        )

    def tearDown(self):
        super(TestBytearrayStream, self).tearDown()

    def test_init(self):
        value = b'\x00'
        b = utils.BytearrayStream(value)

        buf_type = type(b.buffer)
        msg = self.bad_type.format('buffer', type(b''), buf_type)
        self.assertIsInstance(b.buffer, type(b''),
                              msg.format(type(b''), type(b.buffer)))

        length = len(b.buffer)
        msg = self.bad_len.format('buffer', 1, length)
        self.assertEqual(1, length, msg)

        content = b.buffer
        msg = self.bad_val.format('buffer', value, content)
        self.assertEqual(value, content, msg)

    def test_init_unset(self):
        b = utils.BytearrayStream()

        buf_type = type(b.buffer)
        msg = self.bad_type.format('buffer', type(b''), buf_type)
        self.assertIsInstance(b.buffer, type(b''),
                              msg.format(type(b''), type(b.buffer)))

        length = len(b.buffer)
        msg = self.bad_len.format('buffer', 0, length)
        self.assertEqual(0, length, msg)

    def test_read(self):
        # TODO (peter-hamilton) Finish implementation.
        self.skipTest('')

    def test_write(self):
        # TODO (peter-hamilton) Finish implementation.
        self.skipTest('')

    def test_peek(self):
        # TODO (peter-hamilton) Finish implementation.
        value = (b'\x00\x01\x02\x03')
        expected = value
        b = expected
        expected = b
        b = utils.BytearrayStream(value)

    def test_peek_overflow(self):
        # TODO (peter-hamilton) Finish implementation.
        self.skipTest('')

    def test_peek_empty(self):
        # TODO (peter-hamilton) Finish implementation.
        self.skipTest('')

    def test_peek_none(self):
        # TODO (peter-hamilton) Finish implementation.
        self.skipTest('')

    def test_length(self):
        # TODO (peter-hamilton) Finish implementation.
        self.skipTest('')



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import binascii
import logging
import optparse
import sys

from kmip.core.attributes import CryptographicAlgorithm
from kmip.core.attributes import CryptographicLength

from kmip.core.enums import AttributeType
from kmip.core.enums import CertificateType
from kmip.core.enums import CryptographicAlgorithm as CryptoAlgorithmEnum
from kmip.core.enums import CryptographicUsageMask
from kmip.core.enums import ObjectType
from kmip.core.enums import Operation
from kmip.core.enums import SecretDataType

from kmip.core.factories.attributes import AttributeFactory

from kmip.core.misc import KeyFormatType

from kmip.core.objects import KeyBlock
from kmip.core.objects import KeyMaterial
from kmip.core.objects import KeyValue

from kmip.core.secrets import Certificate
from kmip.core.secrets import PrivateKey
from kmip.core.secrets import PublicKey
from kmip.core.secrets import SymmetricKey
from kmip.core.secrets import SecretData


def build_console_logger(level):
    logger = logging.getLogger('demo')
    logger.setLevel(level)
    handler = logging.StreamHandler()
    formatter = logging.Formatter(
        '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    )
    handler.setFormatter(formatter)
    logger.addHandler(handler)
    return logger


def build_cli_parser(operation):
    # Build the argument parser and setup expected options
    parser = optparse.OptionParser(
        usage="%prog [options]",
        description="Run KMIP client operation")

    parser.add_option(
        "-u",
        "--username",
        action="store",
        type="str",
        default=None,
        dest="username",
        help="Username for KMIP server account")
    parser.add_option(
        "-p",
        "--password",
        action="store",
        type="str",
        default=None,
        dest="password",
        help="Password for KMIP server account")
    parser.add_option(
        "-c",
        "--config",
        action="store",
        type="str",
        default="client",
        dest="config",
        help="Client configuration group to load from configuration file")
    parser.add_option(
        "-s",
        "--config-file",
        action="store",
        type="str",
        default=None,
        dest="config_file",
        help="Path to the client configuration file."
    )

    if operation is Operation.CREATE:
        parser.add_option(
            "-a",
            "--algorithm",
            action="store",
            type="str",
            default=None,
            dest="algorithm",
            help="Encryption algorithm for the secret (e.g., AES)")
        parser.add_option(
            "-l",
            "--length",
            action="store",
            type="int",
            default=None,
            dest="length",
            help="Key length in bits (e.g., 128, 256)")
        parser.add_option(
            "-o",
            "--operation-policy-name",
            action="store",
            type="str",
            default=None,
            dest="operation_policy_name",
            help="Operation policy name for the secret (e.g., 'default')"
        )
    elif operation is Operation.CREATE_KEY_PAIR:
        parser.add_option(
            "-a",
            "--algorithm",
            action="store",
            type="str",
            default=None,
            dest="algorithm",
            help="Encryption algorithm for the secret (e.g., RSA)")
        parser.add_option(
            "-l",
            "--length",
            action="store",
            type="int",
            default=None,
            dest="length",
            help="Key length in bits (e.g., 512, 1024, 2048)")
        parser.add_option(
            "-n",
            "--name",
            action="store",
            type="str",
            default=None,
            dest="name",
            help="Name of key pair to create")
        parser.add_option(
            "-o",
            "--operation-policy-name",
            action="store",
            type="str",
            default=None,
            dest="operation_policy_name",
            help="Operation policy name for the secrets (e.g., 'default')"
        )

    elif operation is Operation.DESTROY:
        parser.add_option(
            "-i",
            "--uuid",
            action="store",
            type="str",
            default=None,
            dest="uuid",
            help="UUID of secret to delete from the KMIP server")
    elif operation is Operation.ACTIVATE:
        parser.add_option(
            "-i",
            "--uuid",
            action="store",
            type="str",
            default=None,
            dest="uuid",
            help="UUID of secret to activate on the KMIP server")
    elif operation is Operation.REVOKE:
        parser.add_option(
            "-i",
            "--uuid",
            action="store",
            type="str",
            default=None,
            dest="uuid",
            help="UUID of secret to revoke on the KMIP server")
    elif operation is Operation.GET:
        parser.add_option(
            "-i",
            "--uuid",
            action="store",
            type="str",
            default=None,
            dest="uuid",
            help="UUID of secret to retrieve from the KMIP server")
        parser.add_option(
            "-f",
            "--format",
            action="store",
            type="str",
            default=None,
            dest="format",
            help=("Format in which to retrieve the secret. Supported formats "
                  "include: RAW, PKCS_1, PKCS_8, X_509"))
    elif operation is Operation.GET_ATTRIBUTE_LIST:
        parser.add_option(
            "-i",
            "--uuid",
            action="store",
            type="str",
            default=None,
            dest="uuid",
            help="UID of a managed object")
    elif operation is Operation.GET_ATTRIBUTES:
        parser.add_option(
            "-i",
            "--uuid",
            action="store",
            type="str",
            default=None,
            dest="uuid",
            help="UID of a managed object")
        parser.add_option(
            "-a",
            "--attribute-names",
            action="append",
            type="str",
            default=None,
            dest="attribute_names",
            help="List of attribute names to retrieve, defaults to all "
                 "attributes")
    elif operation is Operation.MODIFY_ATTRIBUTE:
        parser.add_option(
            "-i",
            "--uuid",
            action="store",
            type="str",
            default=None,
            dest="uuid",
            help="UID of a managed object")
    elif operation is Operation.DELETE_ATTRIBUTE:
        parser.add_option(
            "-i",
            "--uuid",
            action="store",
            type="str",
            default=None,
            dest="uuid",
            help="UID of a managed object")
    elif operation is Operation.SET_ATTRIBUTE:
        parser.add_option(
            "-i",
            "--uuid",
            action="store",
            type="str",
            default=None,
            dest="uuid",
            help="UID of a managed object")
    elif operation is Operation.LOCATE:
        parser.add_option(
            "--offset-items",
            action="store",
            type="int",
            default=None,
            dest="offset_items",
            help="The number of matching secrets to skip."
        )
        parser.add_option(
            "--maximum-items",
            action="store",
            type="int",
            default=None,
            dest="maximum_items",
            help="The maximum number of matching secrets to return."
        )
        parser.add_option(
            "-n",
            "--name",
            action="store",
            type="str",
            default=None,
            dest="name",
            help="Name of secret to retrieve from the KMIP server")
        parser.add_option(
            "--initial-date",
            action="append",
            type="str",
            default=[],
            dest="initial_dates",
            help=(
                "Initial date(s) in UTC of the secret to retrieve from the "
                "KMIP server. Use once to perform an exact date match. Use "
                "twice to create a date range that the secret's date should "
                "be within. The value format should look like this: "
                "'Tue Jul 23 18:39:01 2019'"
            )
        )
        parser.add_option(
            "--state",
            action="store",
            type="str",
            default=None,
            dest="state",
            help="The state of the secret (e.g., PRE_ACTIVE, ACTIVE)"
        )
        parser.add_option(
            "--object-type",
            action="store",
            type="str",
            default=None,
            dest="object_type",
            help=(
                "The object type of the secret "
                "(e.g., CERTIFICATE, SYMMETRIC_KEY)"
            )
        )
        parser.add_option(
            "--certificate-type",
            action="store",
            type="str",
            default=None,
            dest="certificate_type",
            help="The certificate type of the secret (e.g., X_509)"
        )
        parser.add_option(
            "--cryptographic-algorithm",
            action="store",
            type="str",
            default=None,
            dest="cryptographic_algorithm",
            help="The cryptographic algorithm of the secret (e.g., AES, RSA)"
        )
        parser.add_option(
            "--cryptographic-length",
            action="store",
            type="int",
            default=None,
            dest="cryptographic_length",
            help="The cryptographic length of the secret (e.g., 128, 2048)"
        )
        parser.add_option(
            "--cryptographic-usage-mask",
            action="append",
            type="str",
            default=[],
            dest="cryptographic_usage_masks",
            help=(
                "The cryptographic usage mask(s) the secret should have set "
                "(e.g., ENCRYPT, DECRYPT). Use multiple times to specify "
                "multiple cryptographic usage mask enumeration values. All "
                "values will get combined into a single mask when sent to the "
                "server."
            )
        )
        parser.add_option(
            "-i",
            "--unique-identifier",
            action="store",
            type="str",
            default=None,
            dest="unique_identifier",
            help="The unique identifier of the secret (e.g., 1, 2, 3)"
        )
        parser.add_option(
            "--operation-policy-name",
            action="store",
            type="str",
            default=None,
            dest="operation_policy_name",
            help="The operation policy name of the secret (e.g., default)"
        )
    elif operation is Operation.REGISTER:
        parser.add_option(
            "-f",
            "--format",
            action="store",
            type="str",
            default="RAW",
            dest="format",
            help=("Format in which to store the secret. Supported formats "
                  "include: RAW, PKCS_1, PKCS_8, X_509"))
        parser.add_option(
            "-t",
            "--type",
            action="store",
            type="str",
            default="SYMMETRIC_KEY",
            dest="type",
            help=("Type of the object to register. Supported types include: "
                  "CERTIFICATE, PRIVATE_KEY, PUBLIC_KEY, SYMMETRIC_KEY, "
                  "SECRET_DATA"))
        parser.add_option(
            "-o",
            "--operation-policy-name",
            action="store",
            type="str",
            default=None,
            dest="operation_policy_name",
            help="Operation policy name for the secret (e.g., 'default')"
        )
    elif operation is Operation.DISCOVER_VERSIONS:
        parser.add_option(
            "-v",
            "--protocol-versions",
            action="store",
            type="str",
            default=None,
            dest="protocol_versions",
            help=("Protocol versions supported by client. "
                  "ex. '1.1,1.2 1.3'"))
    elif operation is Operation.MAC:
        parser.add_option(
            "-i",
            "--uuid",
            action="store",
            type="str",
            default=None,
            dest="uuid",
            help="The unique ID of the managed object that is the key"
                 "to use for the MAC operation")
        parser.add_option(
            "-a",
            "--algorithm",
            action="store",
            type="str",
            default=None,
            dest="algorithm",
            help="Encryption algorithm for the secret (e.g., AES)")
    elif operation is Operation.ENCRYPT:
        parser.add_option(
            "-m",
            "--message",
            action="store",
            type="str",
            default="This is a secret message.",
            dest="message",
            help="The message to encrypt."
        )
    elif operation is Operation.DECRYPT:
        parser.add_option(
            "-i",
            "--uuid",
            action="store",
            type="str",
            default=None,
            dest="uuid",
            help="The unique ID of the decryption key to use to decrypt the "
                 "cipher text."
        )
        parser.add_option(
            "-m",
            "--message",
            action="store",
            type="str",
            default=None,
            dest="message",
            help="The cipher text to decrypt."
        )
    return parser


def build_cryptographic_usage_mask(logger, object_type):
    if object_type == ObjectType.CERTIFICATE:
        flags = [CryptographicUsageMask.ENCRYPT,
                 CryptographicUsageMask.VERIFY]
    elif (object_type == ObjectType.SYMMETRIC_KEY or
          object_type == ObjectType.SECRET_DATA):
        flags = [CryptographicUsageMask.ENCRYPT,
                 CryptographicUsageMask.DECRYPT]
    elif object_type == ObjectType.PUBLIC_KEY:
        flags = [CryptographicUsageMask.VERIFY]
    elif object_type == ObjectType.PRIVATE_KEY:
        flags = [CryptographicUsageMask.SIGN]
    else:
        logger.error("Unrecognized object type, could not build cryptographic "
                     "usage mask")
        sys.exit()

    attribute_type = AttributeType.CRYPTOGRAPHIC_USAGE_MASK
    attribute_factory = AttributeFactory()
    usage_mask = attribute_factory.create_attribute(attribute_type, flags)

    return usage_mask


def build_object(logger, object_type, key_format_type):

    if object_type == ObjectType.CERTIFICATE:
        value = build_secret_value(logger, object_type)
        return Certificate(
            certificate_type=CertificateType.X_509,
            certificate_value=value)
    else:
        return build_key(logger, object_type, key_format_type)


def build_key(logger, object_type, key_format_type):

    key_value = build_secret_value(logger, object_type)
    cryptographic_algorithm = build_cryptographic_algorithm(
        logger, object_type)
    cryptographic_length = build_cryptographic_length(logger, object_type)

    key_block = build_key_block(
        key_format_type,
        key_value,
        cryptographic_algorithm,
        cryptographic_length)

    if object_type == ObjectType.SYMMETRIC_KEY:
        return SymmetricKey(key_block)
    elif object_type == ObjectType.PUBLIC_KEY:
        return PublicKey(key_block)
    elif object_type == ObjectType.PRIVATE_KEY:
        return PrivateKey(key_block)
    elif object_type == ObjectType.SECRET_DATA:
        kind = SecretData.SecretDataType(SecretDataType.PASSWORD)
        return SecretData(secret_data_type=kind,
                          key_block=key_block)
    else:
        logger.error("Unrecognized object type, could not build key")
        sys.exit()


def build_cryptographic_length(logger, object_type):
    if (object_type == ObjectType.SYMMETRIC_KEY or
       object_type == ObjectType.SECRET_DATA):
        return CryptographicLength(128)
    elif object_type == ObjectType.PUBLIC_KEY:
        return CryptographicLength(1024)
    elif object_type == ObjectType.PRIVATE_KEY:
        return CryptographicLength(1024)
    else:
        logger.error("Unrecognized object type, could not build cryptographic "
                     "length")
        sys.exit()


def build_cryptographic_algorithm(logger, object_type):
    if (object_type == ObjectType.SYMMETRIC_KEY or
       object_type == ObjectType.SECRET_DATA):
        return CryptographicAlgorithm(CryptoAlgorithmEnum.AES)
    elif object_type == ObjectType.PUBLIC_KEY:
        return CryptographicAlgorithm(CryptoAlgorithmEnum.RSA)
    elif object_type == ObjectType.PRIVATE_KEY:
        return CryptographicAlgorithm(CryptoAlgorithmEnum.RSA)
    else:
        logger.error("Unrecognized object type, could not build cryptographic "
                     "algorithm")
        sys.exit()


def build_secret_value(logger, object_type):
    if object_type == ObjectType.CERTIFICATE:
        # Encoding from Section 13.2 of the KMIP 1.1 Test Cases document.
        return (
            b'\x30\x82\x03\x12\x30\x82\x01\xFA\xA0\x03\x02\x01\x02\x02\x01\x01'
            b'\x30\x0D\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x05\x05\x00\x30'
            b'\x3B\x31\x0B\x30\x09\x06\x03\x55\x04\x06\x13\x02\x55\x53\x31\x0D'
            b'\x30\x0B\x06\x03\x55\x04\x0A\x13\x04\x54\x45\x53\x54\x31\x0E\x30'
            b'\x0C\x06\x03\x55\x04\x0B\x13\x05\x4F\x41\x53\x49\x53\x31\x0D\x30'
            b'\x0B\x06\x03\x55\x04\x03\x13\x04\x4B\x4D\x49\x50\x30\x1E\x17\x0D'
            b'\x31\x30\x31\x31\x30\x31\x32\x33\x35\x39\x35\x39\x5A\x17\x0D\x32'
            b'\x30\x31\x31\x30\x31\x32\x33\x35\x39\x35\x39\x5A\x30\x3B\x31\x0B'
            b'\x30\x09\x06\x03\x55\x04\x06\x13\x02\x55\x53\x31\x0D\x30\x0B\x06'
            b'\x03\x55\x04\x0A\x13\x04\x54\x45\x53\x54\x31\x0E\x30\x0C\x06\x03'
            b'\x55\x04\x0B\x13\x05\x4F\x41\x53\x49\x53\x31\x0D\x30\x0B\x06\x03'
            b'\x55\x04\x03\x13\x04\x4B\x4D\x49\x50\x30\x82\x01\x22\x30\x0D\x06'
            b'\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x01\x05\x00\x03\x82\x01\x0F'
            b'\x00\x30\x82\x01\x0A\x02\x82\x01\x01\x00\xAB\x7F\x16\x1C\x00\x42'
            b'\x49\x6C\xCD\x6C\x6D\x4D\xAD\xB9\x19\x97\x34\x35\x35\x77\x76\x00'
            b'\x3A\xCF\x54\xB7\xAF\x1E\x44\x0A\xFB\x80\xB6\x4A\x87\x55\xF8\x00'
            b'\x2C\xFE\xBA\x6B\x18\x45\x40\xA2\xD6\x60\x86\xD7\x46\x48\x34\x6D'
            b'\x75\xB8\xD7\x18\x12\xB2\x05\x38\x7C\x0F\x65\x83\xBC\x4D\x7D\xC7'
            b'\xEC\x11\x4F\x3B\x17\x6B\x79\x57\xC4\x22\xE7\xD0\x3F\xC6\x26\x7F'
            b'\xA2\xA6\xF8\x9B\x9B\xEE\x9E\x60\xA1\xD7\xC2\xD8\x33\xE5\xA5\xF4'
            b'\xBB\x0B\x14\x34\xF4\xE7\x95\xA4\x11\x00\xF8\xAA\x21\x49\x00\xDF'
            b'\x8B\x65\x08\x9F\x98\x13\x5B\x1C\x67\xB7\x01\x67\x5A\xBD\xBC\x7D'
            b'\x57\x21\xAA\xC9\xD1\x4A\x7F\x08\x1F\xCE\xC8\x0B\x64\xE8\xA0\xEC'
            b'\xC8\x29\x53\x53\xC7\x95\x32\x8A\xBF\x70\xE1\xB4\x2E\x7B\xB8\xB7'
            b'\xF4\xE8\xAC\x8C\x81\x0C\xDB\x66\xE3\xD2\x11\x26\xEB\xA8\xDA\x7D'
            b'\x0C\xA3\x41\x42\xCB\x76\xF9\x1F\x01\x3D\xA8\x09\xE9\xC1\xB7\xAE'
            b'\x64\xC5\x41\x30\xFB\xC2\x1D\x80\xE9\xC2\xCB\x06\xC5\xC8\xD7\xCC'
            b'\xE8\x94\x6A\x9A\xC9\x9B\x1C\x28\x15\xC3\x61\x2A\x29\xA8\x2D\x73'
            b'\xA1\xF9\x93\x74\xFE\x30\xE5\x49\x51\x66\x2A\x6E\xDA\x29\xC6\xFC'
            b'\x41\x13\x35\xD5\xDC\x74\x26\xB0\xF6\x05\x02\x03\x01\x00\x01\xA3'
            b'\x21\x30\x1F\x30\x1D\x06\x03\x55\x1D\x0E\x04\x16\x04\x14\x04\xE5'
            b'\x7B\xD2\xC4\x31\xB2\xE8\x16\xE1\x80\xA1\x98\x23\xFA\xC8\x58\x27'
            b'\x3F\x6B\x30\x0D\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x05\x05'
            b'\x00\x03\x82\x01\x01\x00\xA8\x76\xAD\xBC\x6C\x8E\x0F\xF0\x17\x21'
            b'\x6E\x19\x5F\xEA\x76\xBF\xF6\x1A\x56\x7C\x9A\x13\xDC\x50\xD1\x3F'
            b'\xEC\x12\xA4\x27\x3C\x44\x15\x47\xCF\xAB\xCB\x5D\x61\xD9\x91\xE9'
            b'\x66\x31\x9D\xF7\x2C\x0D\x41\xBA\x82\x6A\x45\x11\x2F\xF2\x60\x89'
            b'\xA2\x34\x4F\x4D\x71\xCF\x7C\x92\x1B\x4B\xDF\xAE\xF1\x60\x0D\x1B'
            b'\xAA\xA1\x53\x36\x05\x7E\x01\x4B\x8B\x49\x6D\x4F\xAE\x9E\x8A\x6C'
            b'\x1D\xA9\xAE\xB6\xCB\xC9\x60\xCB\xF2\xFA\xE7\x7F\x58\x7E\xC4\xBB'
            b'\x28\x20\x45\x33\x88\x45\xB8\x8D\xD9\xAE\xEA\x53\xE4\x82\xA3\x6E'
            b'\x73\x4E\x4F\x5F\x03\xB9\xD0\xDF\xC4\xCA\xFC\x6B\xB3\x4E\xA9\x05'
            b'\x3E\x52\xBD\x60\x9E\xE0\x1E\x86\xD9\xB0\x9F\xB5\x11\x20\xC1\x98'
            b'\x34\xA9\x97\xB0\x9C\xE0\x8D\x79\xE8\x13\x11\x76\x2F\x97\x4B\xB1'
            b'\xC8\xC0\x91\x86\xC4\xD7\x89\x33\xE0\xDB\x38\xE9\x05\x08\x48\x77'
            b'\xE1\x47\xC7\x8A\xF5\x2F\xAE\x07\x19\x2F\xF1\x66\xD1\x9F\xA9\x4A'
            b'\x11\xCC\x11\xB2\x7E\xD0\x50\xF7\xA2\x7F\xAE\x13\xB2\x05\xA5\x74'
            b'\xC4\xEE\x00\xAA\x8B\xD6\x5D\x0D\x70\x57\xC9\x85\xC8\x39\xEF\x33'
            b'\x6A\x44\x1E\xD5\x3A\x53\xC6\xB6\xB6\x96\xF1\xBD\xEB\x5F\x7E\xA8'
            b'\x11\xEB\xB2\x5A\x7F\x86')
    elif (object_type == ObjectType.SYMMETRIC_KEY or
          object_type == ObjectType.SECRET_DATA):
        return (
            b'\x30\x82\x02\x76\x02\x01\x00\x30\x0D\x06\x09\x2A\x86\x48\x86\xF7'
            b'\x0D\x01\x01\x01\x05\x00\x04\x82\x02\x60\x30\x82\x02\x5C\x02\x01'
            b'\x00\x02\x81\x81\x00\x93\x04\x51\xC9\xEC\xD9\x4F\x5B\xB9\xDA\x17'
            b'\xDD\x09\x38\x1B\xD2\x3B\xE4\x3E\xCA\x8C\x75\x39\xF3\x01\xFC\x8A'
            b'\x8C\xD5\xD5\x27\x4C\x3E\x76\x99\xDB\xDC\x71\x1C\x97\xA7\xAA\x91'
            b'\xE2\xC5\x0A\x82\xBD\x0B\x10\x34\xF0\xDF\x49\x3D\xEC\x16\x36\x24'
            b'\x27\xE5\x8A\xCC\xE7\xF6\xCE\x0F\x9B\xCC\x61\x7B\xBD\x8C\x90\xD0'
            b'\x09\x4A\x27\x03\xBA\x0D\x09\xEB\x19\xD1\x00\x5F\x2F\xB2\x65'
            b'\x52')
    elif object_type == ObjectType.PUBLIC_KEY:
        return (
            b'\x30\x81\x9F\x30\x0D\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x01'
            b'\x05\x00\x03\x81\x8D\x00\x30\x81\x89\x02\x81\x81\x00\x93\x04\x51'
            b'\xC9\xEC\xD9\x4F\x5B\xB9\xDA\x17\xDD\x09\x38\x1B\xD2\x3B\xE4\x3E'
            b'\xCA\x8C\x75\x39\xF3\x01\xFC\x8A\x8C\xD5\xD5\x27\x4C\x3E\x76\x99'
            b'\xDB\xDC\x71\x1C\x97\xA7\xAA\x91\xE2\xC5\x0A\x82\xBD\x0B\x10\x34'
            b'\xF0\xDF\x49\x3D\xEC\x16\x36\x24\x27\xE5\x8A\xCC\xE7\xF6\xCE\x0F'
            b'\x9B\xCC\x61\x7B\xBD\x8C\x90\xD0\x09\x4A\x27\x03\xBA\x0D\x09\xEB'
            b'\x19\xD1\x00\x5F\x2F\xB2\x65\x52\x6A\xAC\x75\xAF\x32\xF8\xBC\x78'
            b'\x2C\xDE\xD2\xA5\x7F\x81\x1E\x03\xEA\xF6\x7A\x94\x4D\xE5\xE7\x84'
            b'\x13\xDC\xA8\xF2\x32\xD0\x74\xE6\xDC\xEA\x4C\xEC\x9F\x02\x03\x01'
            b'\x00\x01')
    elif object_type == ObjectType.PRIVATE_KEY:
        return (
            b'\x30\x82\x02\x76\x02\x01\x00\x30\x0D\x06\x09\x2A\x86\x48\x86\xF7'
            b'\x0D\x01\x01\x01\x05\x00\x04\x82\x02\x60\x30\x82\x02\x5C\x02\x01'
            b'\x00\x02\x81\x81\x00\x93\x04\x51\xC9\xEC\xD9\x4F\x5B\xB9\xDA\x17'
            b'\xDD\x09\x38\x1B\xD2\x3B\xE4\x3E\xCA\x8C\x75\x39\xF3\x01\xFC\x8A'
            b'\x8C\xD5\xD5\x27\x4C\x3E\x76\x99\xDB\xDC\x71\x1C\x97\xA7\xAA\x91'
            b'\xE2\xC5\x0A\x82\xBD\x0B\x10\x34\xF0\xDF\x49\x3D\xEC\x16\x36\x24'
            b'\x27\xE5\x8A\xCC\xE7\xF6\xCE\x0F\x9B\xCC\x61\x7B\xBD\x8C\x90\xD0'
            b'\x09\x4A\x27\x03\xBA\x0D\x09\xEB\x19\xD1\x00\x5F\x2F\xB2\x65\x52'
            b'\x6A\xAC\x75\xAF\x32\xF8\xBC\x78\x2C\xDE\xD2\xA5\x7F\x81\x1E\x03'
            b'\xEA\xF6\x7A\x94\x4D\xE5\xE7\x84\x13\xDC\xA8\xF2\x32\xD0\x74\xE6'
            b'\xDC\xEA\x4C\xEC\x9F\x02\x03\x01\x00\x01\x02\x81\x80\x0B\x6A\x7D'
            b'\x73\x61\x99\xEA\x48\xA4\x20\xE4\x53\x7C\xA0\xC7\xC0\x46\x78\x4D'
            b'\xCB\xEA\xA6\x3B\xAE\xBC\x0B\xC1\x32\x78\x74\x49\xCD\xE8\xD7\xCA'
            b'\xD0\xC0\xC8\x63\xC0\xFE\xFB\x06\xC3\x06\x2B\xEF\xC5\x00\x33\xEC'
            b'\xF8\x7B\x4E\x33\xA9\xBE\x7B\xCB\xC8\xF1\x51\x1A\xE2\x15\xE8\x0D'
            b'\xEB\x5D\x8A\xF2\xBD\x31\x31\x9D\x78\x21\x19\x66\x40\x93\x5A\x0C'
            b'\xD6\x7C\x94\x59\x95\x79\xF2\x10\x0D\x65\xE0\x38\x83\x1F\xDA\xFB'
            b'\x0D\xBE\x2B\xBD\xAC\x00\xA6\x96\xE6\x7E\x75\x63\x50\xE1\xC9\x9A'
            b'\xCE\x11\xA3\x6D\xAB\xAC\x3E\xD3\xE7\x30\x96\x00\x59\x02\x41\x00'
            b'\xDD\xF6\x72\xFB\xCC\x5B\xDA\x3D\x73\xAF\xFC\x4E\x79\x1E\x0C\x03'
            b'\x39\x02\x24\x40\x5D\x69\xCC\xAA\xBC\x74\x9F\xAA\x0D\xCD\x4C\x25'
            b'\x83\xC7\x1D\xDE\x89\x41\xA7\xB9\xAA\x03\x0F\x52\xEF\x14\x51\x46'
            b'\x6C\x07\x4D\x4D\x33\x8F\xE6\x77\x89\x2A\xCD\x9E\x10\xFD\x35\xBD'
            b'\x02\x41\x00\xA9\x8F\xBC\x3E\xD6\xB4\xC6\xF8\x60\xF9\x71\x65\xAC'
            b'\x2F\x7B\xB6\xF2\xE2\xCB\x19\x2A\x9A\xBD\x49\x79\x5B\xE5\xBC\xF3'
            b'\x7D\x8E\xE6\x9A\x6E\x16\x9C\x24\xE5\xC3\x2E\x4E\x7F\xA3\x32\x65'
            b'\x46\x14\x07\xF9\x52\xBA\x49\xE2\x04\x81\x8A\x2F\x78\x5F\x11\x3F'
            b'\x92\x2B\x8B\x02\x40\x25\x3F\x94\x70\x39\x0D\x39\x04\x93\x03\x77'
            b'\x7D\xDB\xC9\x75\x0E\x9D\x64\x84\x9C\xE0\x90\x3E\xAE\x70\x4D\xC9'
            b'\xF5\x89\xB7\x68\x0D\xEB\x9D\x60\x9F\xD5\xBC\xD4\xDE\xCD\x6F\x12'
            b'\x05\x42\xE5\xCF\xF5\xD7\x6F\x2A\x43\xC8\x61\x5F\xB5\xB3\xA9\x21'
            b'\x34\x63\x79\x7A\xA9\x02\x41\x00\xA1\xDD\xF0\x23\xC0\xCD\x94\xC0'
            b'\x19\xBB\x26\xD0\x9B\x9E\x3C\xA8\xFA\x97\x1C\xB1\x6A\xA5\x8B\x9B'
            b'\xAF\x79\xD6\x08\x1A\x1D\xBB\xA4\x52\xBA\x53\x65\x3E\x28\x04\xBA'
            b'\x98\xFF\x69\xE8\xBB\x1B\x3A\x16\x1E\xA2\x25\xEA\x50\x14\x63\x21'
            b'\x6A\x8D\xAB\x9B\x88\xA7\x5E\x5F\x02\x40\x61\x78\x64\x6E\x11\x2C'
            b'\xF7\x9D\x92\x1A\x8A\x84\x3F\x17\xF6\xE7\xFF\x97\x4F\x68\x81\x22'
            b'\x36\x5B\xF6\x69\x0C\xDF\xC9\x96\xE1\x89\x09\x52\xEB\x38\x20\xDD'
            b'\x18\x90\xEC\x1C\x86\x19\xE8\x7A\x2B\xD3\x8F\x9D\x03\xB3\x7F\xAC'
            b'\x74\x2E\xFB\x74\x8C\x78\x85\x94\x2C\x39')
    else:
        logger.error("Unrecognized object type, could not build secret value")
        sys.exit()


def build_key_block(key_format_type, key_value, cryptographic_algorithm,
                    cryptographic_length):
    key_material = KeyMaterial(key_value)
    key_value = KeyValue(key_material)

    return KeyBlock(
        key_format_type=KeyFormatType(key_format_type),
        key_compression_type=None,
        key_value=key_value,
        cryptographic_algorithm=cryptographic_algorithm,
        cryptographic_length=cryptographic_length,
        key_wrapping_data=None)


def log_template_attribute(logger, template_attribute):
    names = template_attribute.names
    attributes = template_attribute.attributes

    logger.info('number of template attribute names: {0}'.format(len(names)))
    for i in range(len(names)):
        name = names[i]
        logger.info('name {0}: {1}'.format(i, name))

    log_attribute_list(logger, attributes)


def log_attribute_list(logger, attributes):
    logger.info('number of attributes: {0}'.format(len(attributes)))
    for i in range(len(attributes)):
        attribute = attributes[i]
        attribute_name = attribute.attribute_name
        attribute_index = attribute.attribute_index
        attribute_value = attribute.attribute_value

        logger.info('attribute {0}:'.format(i))
        logger.info('   attribute_name: {0}'.format(attribute_name))
        logger.info('   attribute_index: {0}'.format(attribute_index))
        logger.info('   attribute_value: {0}'.format(
            repr(attribute_value)))


def log_secret(logger, secret_type, secret_value):
    if secret_type is ObjectType.CERTIFICATE:
        log_certificate(logger, secret_value)
    elif secret_type is ObjectType.PRIVATE_KEY:
        log_private_key(logger, secret_value)
    elif secret_type is ObjectType.PUBLIC_KEY:
        log_public_key(logger, secret_value)
    elif secret_type is ObjectType.SYMMETRIC_KEY:
        log_symmetric_key(logger, secret_value)
    elif secret_type is ObjectType.SPLIT_KEY:
        log_split_key(logger, secret_value)
    else:
        logger.info('generic secret: {0}'.format(secret_value))


def log_certificate(logger, certificate):
    logger.info('certificate:')
    logger.info('* certificate type: {0}'.format(
        certificate.certificate_type))
    logger.info('* certificate value: {0}'.format(
        binascii.hexlify(certificate.certificate_value.value)))


def log_public_key(logger, public_key):
    key_block = public_key.key_block

    log_key_block(logger, key_block)


def log_private_key(logger, private_key):
    key_block = private_key.key_block

    log_key_block(logger, key_block)


def log_symmetric_key(logger, skey):
    key_block = skey.key_block

    log_key_block(logger, key_block)


def log_split_key(logger, split_key):
    logger.info("Split Key:")
    logger.info("* Split Key Parts: {}".format(split_key.split_key_parts))
    logger.info(
        "* Key Part Identifier: {}".format(split_key.key_part_identifier)
    )
    logger.info(
        "* Split Key Threshold: {}".format(split_key.split_key_threshold)
    )
    logger.info("* Split Key Method: {}".format(split_key.split_key_method))
    logger.info("* Prime Field Size: {}".format(split_key.prime_field_size))
    log_key_block(logger, split_key.key_block)


def log_key_block(logger, key_block):
    if key_block is not None:
        logger.info("* Key Block:")

        key_format_type = key_block.key_format_type
        key_compression_type = key_block.key_compression_type
        key_value = key_block.key_value
        cryptographic_algorithm = key_block.cryptographic_algorithm
        cryptographic_length = key_block.cryptographic_length
        key_wrapping_data = key_block.key_wrapping_data

        logger.info("  * Key Format Type: {}".format(key_format_type))
        logger.info("  * Key Compression Type: {}".format(
            key_compression_type))
        logger.info("  * Cryptographic Algorithm: {}".format(
            cryptographic_algorithm))
        logger.info("  * Cryptographic Length: {}".format(
            cryptographic_length))
        logger.info("  * Key Wrapping Data: {}".format(key_wrapping_data))

        log_key_value(logger, key_value)
    else:
        logger.info("* Key Block: {}".format(key_block))


def log_key_value(logger, key_value):
    if key_value is not None:
        logger.info("  * Key Value:")
        logger.info(
            "    * Key Material: {}".format(repr(key_value.key_material))
        )
    else:
        logger.info("  * Key Value: {}".format(key_value))



# Copyright (c) 2018 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc
import six


@six.add_metaclass(abc.ABCMeta)
class AuthAPI:
    """
    The base class for an authentication API connector.
    """

    @abc.abstractmethod
    def authenticate(self,
                     connection_certificate=None,
                     connection_info=None,
                     request_credentials=None):
        """
        Query the configured authentication service with the given credentials.

        Args:
            connection_certificate (cryptography.x509.Certificate): An X.509
                certificate object obtained from the connection being
                authenticated. Optional, defaults to None.
            connection_info (tuple): A tuple of information pertaining to the
                connection being authenticated, including the source IP address
                and a timestamp (e.g., ('127.0.0.1', 1519759267.467451)).
                Optional, defaults to None.
            request_credentials (list): A list of KMIP Credential structures
                containing credential information to use for authentication.
                Optional, defaults to None.
        """
